import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.sql.*;
import java.util.*;
/**
 * Implementamos metodos abstractos de AbstractTableModel
 * Sobre-escribimos metodos de AbstractTableModel para customizar
 * JTable
 * 
 * @author G. Cherencio
 * @version 1.0
 */
public class ClienteModel2  extends AbstractTableModel {
    private ResultSet rs = null;
    private List<Cliente> lst = new ArrayList<Cliente>();
    public ClienteModel2(ResultSet r) {
        rs = r;
        try {
            while(rs.next()) {
                lst.add(new Cliente(rs.getInt("idc"),rs.getString("descr"),rs.getDouble("saldo")));
            }
            rs.close();
        } catch(SQLException sqle) {
            System.err.println("ClienteModel(): Error "+sqle.getMessage());
        }
    }
    public int getRowCount() {
        return lst.size();
    }
    public int getColumnCount() {
        return 3;
    }
    public Object getValueAt(int row, int column) {
        Cliente c = lst.get(row);
        switch(column) {
            case 0:
                return new Integer(c.getIdc());
            case 1:
                return c.getDescripcion();
            case 2:
                return new Double(c.getSaldo());
        }
        return ""+row+","+column;
    }
    @Override
    public String getColumnName(int column) {
        switch(column) {
            case 0:
                return "Codigo";
            case 1:
                return "Descripcion";
            case 2:
                return "Saldo";
        }
        return "";
    }
    @Override
    public Class<?> getColumnClass(int columnIndex) {
        switch(columnIndex) {
            case 0:
                return Integer.class;
            case 1:
                return String.class;
            case 2:
                return Double.class;
        }
        return Object.class;
    }
    @Override
    public boolean isCellEditable(int rowIndex,int columnIndex) {
        switch(columnIndex) {
            case 0:
                return false;
            case 1:
                return true;
            case 2:
                return false;
        }
        return false;
    }
    @Override
    public void setValueAt(Object aValue,int rowIndex,int columnIndex) {
        Cliente c = lst.get(rowIndex);
        if ( columnIndex == 1 ) { // permito editar solamente la columna 1 descripcion
            String sdesc = (String) aValue;
            if ( DataIO.executeQuery("UPDATE TBL_CLIENTE SET DESCR = '"+sdesc+"' WHERE IDC = "+c.getIdc()) ) {
                if ( DataIO.doCommit() ) {
                    c.setDescripcion(sdesc);
                    return;
                }
            }
            JOptionPane.showMessageDialog(null, "Error ["+DataIO.getErrorMessages()+"] al intentar modificar Cliente!", "Error BD", JOptionPane.ERROR_MESSAGE);
        }
    }
    
    // METODOS PROPIOS
    public void agregar() {
        try {
            ResultSet rs = DataIO.getResultSet("SELECT COALESCE(MAX(IDC)+1,1) FROM TBL_CLIENTE");
            if ( rs.next() ) {
                int idc = rs.getInt(1);
                rs.close();
                if ( DataIO.executeQuery("INSERT INTO TBL_CLIENTE(IDC,DESCR,SALDO) VALUES("+idc+",'NONE',0.0)") ) {
                    if ( DataIO.doCommit() ) {
                        JOptionPane.showMessageDialog(null, "Cliente dado de Alta Ok!", "Info", JOptionPane.INFORMATION_MESSAGE);
                        lst.add(new Cliente(idc,"NONE",0.0));
                        fireTableRowsInserted(lst.size()-1,lst.size()-1);
                        return;
                    }
                }
            }
            rs.close();
        } catch(SQLException sqle) {
        }        
        JOptionPane.showMessageDialog(null, "Error ["+DataIO.getErrorMessages()+"] al intentar dar de alta Cliente!", "Error BD", JOptionPane.ERROR_MESSAGE);
    }
    public void borrar(int rowIndex) {
        Cliente c = lst.get(rowIndex);
        if ( JOptionPane.showConfirmDialog(null,"¿Borro al Cliente ["+c.getDescripcion()+"] ?", "Confirme Baja",
            JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE) == JOptionPane.YES_OPTION ) {
            if ( DataIO.executeQuery("DELETE FROM TBL_CLIENTE WHERE IDC = "+c.getIdc()) ) {
                if ( DataIO.doCommit() ) {
                    JOptionPane.showMessageDialog(null, "Cliente dado de Baja Ok!", "Info", JOptionPane.INFORMATION_MESSAGE);
                    lst.remove(rowIndex);
                    fireTableRowsDeleted(rowIndex,rowIndex);
                    return;
                }
            }
            JOptionPane.showMessageDialog(null, "Error ["+DataIO.getErrorMessages()+"] al intentar dar de baja Cliente!", "Error BD", JOptionPane.ERROR_MESSAGE);
        }
    }
}
