import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.text.*;
import java.util.*;
import java.sql.*;
import java.text.*;

/**
 * Clase de utileria para apoyar a tareas repetitivas de Swing
 * 
 * @author G. Cherencio
 * @version 1.0
 */
public class SwingUtil<E> {
    public static void errMsg(final String msg) {
        errMsg(msg,"Error");
    }
    public static void errMsg(final String msg,final JComponent txt) {
        errMsg(msg,"Error",txt);
    }
    public static void errMsg(final String msg,final String titulo) {
        errMsg(msg,titulo,null);
    }
    public static void errMsg(final String msg,final String titulo,final JComponent txt) {
        JOptionPane.showMessageDialog(null,msg,titulo,JOptionPane.ERROR_MESSAGE);
        if ( txt != null ) txt.requestFocus();
    }
    // validaciones basicas de datos
    /**
     * Valida que el ingreso sea un numero Double mayor que cero
     * @return null si el ingreso es incorrecto, sino devuelve objeto Double
     */
    public static Double chkDouble(final String msg,final JTextComponent txt) {
        Double d = null;
        try {
            d = new Double(txt.getText());
            if ( d < 0 ) {
                errMsg(msg,txt);
                return null;
            }
        }catch(NumberFormatException nfe) {
            errMsg("["+txt.getText()+"] no es un numero!",txt);
            return null;
        }
        return d;
    }
    /**
     * Valida que el ingreso de texto tenga al menos N caracteres
     * @return true si el ingreso es incorrecto, sino devuelve false
     */
    public static boolean chkText(final String msg,final JTextComponent txt,final int length) {
        if ( txt.getText().length() < length ) {
            errMsg(msg,txt);
            return false;
        } else return true;
    }
    /**
     * Devuelve una combo de tipo E, requiere la creacion de un objeto SwingUtil 
     * para el tipo E
     * Creo combo de tipo E a partir de lista de tipo E
     */
    public JComboBox<E> cmb(List<E> l) {
        MyComboModel<E> pm = new MyComboModel<E>(l);
        JComboBox<E> cp = new JComboBox<E>();
        cp.setModel(pm);
        return cp;
    }
    /**
     * Creo combo de productos a partir de lista de productos
     */
    public static JComboBox<Producto> cmbProducto(List<Producto> l) {
        MyComboModel<Producto> pm = new MyComboModel<Producto>(l);
        JComboBox<Producto> cp = new JComboBox<Producto>();
        cp.setModel(pm);
        return cp;
    }
    /**
     * Creo combo de clientes a partir de lista de clientes
     */
    public static JComboBox<Cliente> cmbCliente(List<Cliente> l) {
        MyComboModel<Cliente> pm = new MyComboModel<Cliente>(l);
        JComboBox<Cliente> cp = new JComboBox<Cliente>();
        cp.setModel(pm);
        return cp;
    }
    /**
     * Setea los anchos minimos, maximos y ancho de una columna determinada de una tabla
     */
    public static void setColumnWidth(JTable tbl,int columnIndex,int maxWidth) {
        setColumnWidth(tbl,columnIndex,maxWidth,maxWidth);
    }
    /**
     * Setea los anchos minimos, maximos y ancho de una columna determinada de una tabla
     */
    public static void setColumnWidth(JTable tbl,int columnIndex,int maxWidth,int minWidth) {
        setColumnWidth(tbl,columnIndex,maxWidth,minWidth,maxWidth);
    }
    /**
     * Setea los anchos minimos, maximos y ancho de una columna determinada de una tabla
     */
    public static void setColumnWidth(JTable tbl,int columnIndex,int maxWidth,int minWidth,int width) {
        setColumnWidth(tbl,columnIndex,maxWidth,minWidth,width,width);
    }
    /**
     * Setea los anchos minimos, maximos y ancho de una columna determinada de una tabla
     */
    public static void setColumnWidth(JTable tbl,int columnIndex,int maxWidth,int minWidth,int width,int prefWidth) {
        tbl.getColumnModel().getColumn(columnIndex).setMinWidth(minWidth);
        tbl.getColumnModel().getColumn(columnIndex).setMaxWidth(maxWidth);
        tbl.getColumnModel().getColumn(columnIndex).setWidth(width);        
        tbl.getColumnModel().getColumn(columnIndex).setPreferredWidth(prefWidth);        
    }
    /**
     * Setea el editor de celdas de la columna que se indique sobre la tabla que se indique
     * @param tbl JTable sobre la cual setearemos editor de celdas (el mismo para todas las columnas)
     * @param columnIndex numero de columna de tbl sobre la cual seteamos editor de celdas (indice de base cero)
     * @param te objeto de tipo TableCellEditor, el editor de celdas a utilizar para esta columna
     */
    public static void setColumnEditor(JTable tbl,int columnIndex,TableCellEditor te) {
        TableColumn colpr = tbl.getColumnModel().getColumn(columnIndex);
        colpr.setCellEditor(te);        
    }
    /**
     * Setea el render de celdas de la columna que se indique sobre la tabla que se indique
     * @param tbl JTable sobre la cual setearemos editor de celdas (el mismo para todas las columnas)
     * @param columnIndex numero de columna de tbl sobre la cual seteamos editor de celdas (indice de base cero)
     * @param te objeto de tipo TableCellEditor, el editor de celdas a utilizar para esta columna
     */
    public static void setColumnRenderer(JTable tbl,int columnIndex,TableCellRenderer te) {
        TableColumn colpr = tbl.getColumnModel().getColumn(columnIndex);
        colpr.setCellRenderer(te);        
    }
}

