import java.util.*;

/**
 * Thread utilizado para la actualizacion de las combos de productos y clientes
 * Mientras el usuario esta facturando, pueden producirse altas y bajas sobre
 * clientes y productos, por lo tanto, las combos deben ser actualizadas
 * En swing tenemos 3 tipos de hilos: hilos inciales, hilos de eventos, hilos
 * de background (ver tutorial https://docs.oracle.com/javase/tutorial/uiswing/concurrency/index.html )
 * Este hilo lanza un hilo de background para actualizar los componentes swing
 * que corresponda.
 * 
 * @author G. Cherencio
 * @version 1.0
 * @see https://docs.oracle.com/javase/tutorial/uiswing/concurrency/index.html
 */
public class ThrFactura implements Runnable {
    private MyComboBoxRenderer<Producto> crp = null;
    private MyComboModel<Cliente> cmc = null;
    private boolean continuo = true;
    private static ThrFactura obj = null; // comenzamos con obj en nulo
    /**
     * Constructor for objects of class ThrFactura
     */
    public ThrFactura() {
        control();
    }
    public ThrFactura(MyComboBoxRenderer<Producto> rp,MyComboModel<Cliente> mc) {
        if ( control() ) {
            // initialise instance variables
            crp = rp; cmc = mc;
        }
    }
    private boolean control() {
        if ( obj != null ) {
            System.err.println("ThrFactura.ThrFactura(): Error! pretende crear mas de un objeto ThrFactura!");
            return  false;
        } else {
            obj = this;
            return true;
        }
    }
    
    @Override
    public void run() {
        System.out.println(System.currentTimeMillis()+" ThrFactura().run(): inicio metodo run!");
        if ( crp == null || cmc == null ) continuo=false;
        while(continuo) {
            System.out.println(System.currentTimeMillis()+" ThrFactura().run(): continuo metodo run!");
            // hago proceso
            // obtengo listas
            DataIO.doCommit();
            List<Producto> lp = Data.getProductos();
            List<Cliente> lc = Data.getClientes();
            // comparo las listas
            List<Producto> rlp = crp.getList();
            List<Cliente> lcmc = cmc.getList();
            boolean cambio=false;
            if ( rlp.size() != lp.size() || !rlp.containsAll(lp)) {
                // cambio la lista de Producto
                //System.out.println(System.currentTimeMillis()+" ThrFactura().run(): cambio la lista de productos!");
                //crp.setList(lp);
                cambio=true;
            }
            if ( lcmc.size() != lc.size() || !lcmc.containsAll(lc)) {
                // cambio la lista de Clientes
                //System.out.println(System.currentTimeMillis()+" ThrFactura().run(): cambio la lista de clientes!");
                //cmc.setList(lc);
                cambio=true;
            }
            if ( cambio ) {
                MySwingWorker sw = new MySwingWorker(lp,lc,crp,cmc);
                sw.execute();
            }
            
            System.out.println(System.currentTimeMillis()+" ThrFactura().run(): espero 15 segundos sin hacer nada!");
            try {
                Thread.sleep(15000); // duermo, espero por 15 segundos
            }catch(Exception e) {
                continuo=false;
            }
        }
        System.out.println(System.currentTimeMillis()+" ThrFactura().run(): fin metodo run!");
    }
    
    // SETTERS
    public void setContinuo(boolean b) { continuo = b; }
    public boolean getContinuo()       { return continuo; }
    
    /**
     * Metodo para detener el unico obj de esta clase
     */
    public static void finalizo() {
        if ( obj != null ) {
            obj.setContinuo(false);
            obj=null;
        }
    }

}
